% ReadMe.m

% two main routines are:
% [clutter_levels, clutter_map] = colorClutter(inputImage, numlevels,
% pool_sigma)
%       which computes the color clutter
% and
% [clutter_levels, clutter_map] = contrastClutter(inputImage, numlevels,
% filt_sigma, pool_sigma)
%       which computes the contrast clutter
%

% Reference:
% Ruth Rosenholtz, Yuanzhen Li, Jonathan Mansfield, and Zhenlan Jin. 
% Feature Congestion: A Measure of Display Clutter. CHI '05: Proc. of the
% SIGCHI conference 
% on Human factors in computing systems. May 2005. 761-770.  


% Specifics and examples:

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [clutter_levels, clutter_map] = colorClutter(inputImage, numlevels,
% pool_sigma)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   computes the color clutter map(s) of an image. Returns:
%   clutter_levels, a cell structure, containing the color clutter at a 
%     number of scales specified by numlevels;  -- cell(numlevels,1) --,
%     the n'th 
%     level of which can be accessed using clutter_levels{n}{1}
%   clutter_map, an array of the same size as inputImage, is a single
%   clutter map 
%     collapsed from clutter_levels, which is the clutter measure at
%     multiple scales
%     now the "collapsing" is done by taking the maximal values across
%     scales
%%%%%%
%   input:
%   inputImage specifies the input image. It can be either a string, in
%   which
%     case it specifies a filename, or an array, in which case it specifies
%     an
%     image array
%   numlevels is the number of levels.
%   Color clutter is computed as the "volume" of a color distribution
%     ellipsoid, which is the determinant of covariance matrix. Covariance 
%     matrix can be computed efficiently through linear filtering. More 
%     specifically, cov(X,Y) = E(XY)-E(X)E(Y), where E (expectation value) 
%     can be approximated by filtering with a Gaussian window. 
%   pool_sigma is the sigma (standard deviation) of this Gaussian window. 
%%%%%%
% example:
[color_clutter_levels, color_clutter_map] = colorClutter('map.jpg', 3, 3);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [clutter_levels, clutter_map] = contrastClutter(inputImage, numlevels,
% filt_sigma, pool_sigma)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   computes the contrast clutter map(s) of an image. Returns:
%   clutter_levels and clutter_map, same as in colorClutter, but containing
%     the contrast clutter maps instead of the color ones
%%%%%% 
%   input:
%   inputImage and numlevels, same as in colorClutter
%   filt_sigma is the sigma of the center-surround DoG1 filter used for
%     computing the contrast 
%   contrast clutter is then computed as the sqrt(variance) of contrast
%   over a local
%     window. The variance can be computed efficiently through linear
%     filtering. More specifically, var(X) = E(X.^2)-E(X).^2, where E
%     (expectation value) 
%     can be approximated by filtering with a Gaussian window. 
%   pool_sigma (optional) is the sigma of this Gaussian window. Default =
%     3*filt_sigma. 
%%%%%%
% example:
[contrast_clutter_levels, contrast_clutter_map] = contrastClutter('map.jpg', 3, 1);

% Referred outside sources:
% upConv.m, upConv.dll, and rConv2.m
% come from Eero Simoncelli's matlab pyramid toolbox "matlabPyrTools":
% http://www.cns.nyu.edu/~lcv/software.html

% Potential warnings:
% upConv.m is not actually used, intead the pre-compiled C version
% upConv.dll is used. This pre-compiled C version has been tested on
% Windows, 
% but there's no guarantee it will work on other platforms. 
% So if it gives an error, remove or rename upConv.dll, and upConv.m will
% be used instead. It will be slower than upConv.dll, but shouldn't matter
% too much. If you're speed concious, please compile a C version yourself.
% 
